@@ -16,6 +16,9 @@ use Gscan2pdf::Scanner::Options;
 use Gscan2pdf::Scanner::Profile;
 use Gscan2pdf::Translation '__';    # easier to extract strings with xgettext
 use List::MoreUtils qw(first_index);
+use Readonly;
+Readonly my $PAPER_TOLERANCE  => 1;
+Readonly my $OPTION_TOLERANCE => 0.001;
 
 my (
     $MAX_PAGES,        $MAX_INCREMENT, $DOUBLE_INCREMENT,
@@ -270,10 +273,9 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
     ),
   ];
 
-our $VERSION = '2.5.4';
+our $VERSION = '2.5.5';
 
 my ( $d_sane, $logger );
-my $tolerance             = 1;
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
 my $SANE_NAME_SCAN_BR_X   = SANE_NAME_SCAN_BR_X;
@@ -573,6 +575,37 @@ sub INIT_INSTANCE {
     $self->{setting_profile}              = [];
     $self->{setting_current_scan_options} = [];
 
+    # If we get an error opening a device, add it to a session-only blacklist,
+    # removing it from the device list
+    $self->{device_blacklist} = [];
+    $self->signal_connect(
+        'process-error' => sub {
+            my ( $widget, $process, $msg ) = @_;
+            if ( $process =~
+                /^(?:get_devices|open_device|find_scan_options)$/xsm )
+            {
+                my $device = $widget->get('device');
+                if ( defined $device ) {
+                    $logger->warn(
+                        "adding device '$device' to session blacklist");
+                    push @{ $self->{device_blacklist} }, $device;
+                    my $device_list = $widget->get('device-list');
+                    my @device_list;
+                    for my $dev ( @{$device_list} ) {
+                        my $found = FALSE;
+                        for ( @{ $self->{device_blacklist} } ) {
+                            if ( $_ eq $dev->{name} ) {
+                                $found = TRUE;
+                                last;
+                            }
+                        }
+                        if ( not $found ) { push @device_list, $dev }
+                    }
+                    $widget->set( 'device-list', \@device_list );
+                }
+            }
+        }
+    );
     return $self;
 }
 
@@ -1205,7 +1238,8 @@ sub create_paper_widget {
             }
         );
 
-        # If the geometry is changed, unset the paper size
+        # If the geometry is changed, unset the paper size,
+        # if we are not setting a profile
         for (
             ( SANE_NAME_SCAN_TL_X, SANE_NAME_SCAN_TL_Y,
                 SANE_NAME_SCAN_BR_X,   SANE_NAME_SCAN_BR_Y,
@@ -1217,7 +1251,9 @@ sub create_paper_widget {
                 my $widget = $self->{option_widgets}{$_};
                 $widget->signal_connect(
                     changed => sub {
-                        if ( defined $self->get('paper') ) {
+                        if ( not @{ $self->{setting_current_scan_options} }
+                            and defined $self->get('paper') )
+                        {
                             $self->set( 'paper', undef );
                         }
                     }
@@ -1450,7 +1486,8 @@ sub set_paper_formats {
         $self->{ignored_paper_formats} = ();
         my $options = $self->get('available-scan-options');
         for ( keys %{$formats} ) {
-            if ( $options->supports_paper( $formats->{$_}, $tolerance ) ) {
+            if ( $options->supports_paper( $formats->{$_}, $PAPER_TOLERANCE ) )
+            {
                 $logger->debug("Options support paper size '$_'.");
                 $combobp->prepend_text($_);
             }
@@ -1539,7 +1576,6 @@ sub set_paper {
                 $self->{paper} = $paper;
                 $self->{current_scan_options}
                   ->add_frontend_option( 'paper', $paper );
-                $self->set( 'profile', undef );
                 $self->signal_emit( 'changed-paper', $paper );
             }
         }
@@ -1739,6 +1775,7 @@ sub save_current_profile {
     $self->{combobsp}->set_active( $self->{combobsp}->get_num_rows - 1 );
     $self->{combobsp}
       ->signal_handler_unblock( $self->{combobsp_changed_signal} );
+    $self->{profile} = $name;
     return;
 }
 
@@ -2207,6 +2244,10 @@ sub add_current_scan_options {
     # First clone the profile, as otherwise it would be self-modifying
     my $clone = dclone($profile);
 
+    # forget the previous option info calls, as these are only interesting
+    # *whilst* setting a profile, and now we are starting from scratch
+    delete $self->{option_info};
+
     push @{ $self->{setting_current_scan_options} }, $clone->{uuid};
 
     # Give the GUI a chance to catch up between settings,
@@ -2245,11 +2286,20 @@ sub _set_option_profile {
         if ( defined $self->{option_info}{ $opt->{name} }
             and $self->{option_info}{ $opt->{name} } & SANE_INFO_INEXACT )
         {
+            $logger->warn(
+"Skip setting option '$name' to '$val', as previous call set SANE_INFO_INEXACT"
+            );
+            $self->_set_option_profile( $profile, $next );
             return;
         }
 
         # Ignore option if value already within tolerance
-        if ( Gscan2pdf::Scanner::Options::within_tolerance( $opt, $val ) ) {
+        if (
+            Gscan2pdf::Scanner::Options::within_tolerance(
+                $opt, $val, $OPTION_TOLERANCE
+            )
+          )
+        {
             $logger->info(
                 "No need to set option '$name': already within tolerance.");
             $self->_set_option_profile( $profile, $next );
